<?php
/*
  Plugin Name: ImageFlux
  Plugin URI:
  Description: WordPressサイトの画像を一括で最適化・軽量化し、CDNを通して配信します。
  Version: 1.0.1
  Author: SAKURA Internet Inc.
  Author URI: https://www.sakura.ad.jp/services/imageflux/
  License: GPLv2
 */

// Load plugin files.
require_once __DIR__ . '/models/ImagefluxSetting.php';
require_once __DIR__ . '/models/ImagefluxBasicSetting.php';
require_once __DIR__ . '/models/ImagefluxDiffSetting.php';
require_once __DIR__ . '/models/ImageFluxOrigin.php';
require_once __DIR__ . '/classes/ImageFlux.php';
require_once __DIR__ . '/classes/TransferCalc.php';

// 管理画面のメニュー
add_action('init', 'ImageFluxManagement::init');
// 記事ページの加工
add_filter('wp_lazy_loading_enabled', '__return_false');
// srcset無効化
add_filter('wp_calculate_image_srcset_meta', '__return_null');
// 画像置換処理
add_filter('the_content', 'ImageFluxPage::doProcess');

/**
 * 管理画面のメニュー
 */
class ImageFluxManagement
{
    public const VERSION = '1.0.0';
    public const PLUGIN_ID = 'image-flux';

    public const CREDENTIAL_ACTION_BASIC = self::PLUGIN_ID . '-nonce-action-basic';
    public const CREDENTIAL_NAME_BASIC = self::PLUGIN_ID . '-nonce-name-basic';
    public const CREDENTIAL_ACTION_UPDATE_BASIC = self::PLUGIN_ID . '-nonce-action-update-basic';
    public const CREDENTIAL_NAME_UPDATE_BASIC = self::PLUGIN_ID . '-nonce-name-update-basic';
    public const CREDENTIAL_ACTION_DIFF = self::PLUGIN_ID . '-nonce-action-diff';
    public const CREDENTIAL_NAME_DIFF = self::PLUGIN_ID . '-nonce-name-diff';
    public const CREDENTIAL_ACTION_ORIGIN_SELECT = self::PLUGIN_ID . '-nonce-action-origin-select';
    public const CREDENTIAL_NAME_ORIGIN_SELECT = self::PLUGIN_ID . '-nonce-name-origin-select';
    public const CREDENTIAL_ACTION_ORIGIN_DELETE = self::PLUGIN_ID . '-nonce-action-origin-delete';
    public const CREDENTIAL_NAME_ORIGIN_DELETE = self::PLUGIN_ID . '-nonce-name-origin-delete';
    public const CREDENTIAL_ACTION_TOKEN = self::PLUGIN_ID . '-nonce-action-token';
    public const CREDENTIAL_NAME_TOKEN = self::PLUGIN_ID . '-nonce-name-token';

    public const COMPLETE_CONFIG = self::PLUGIN_ID . '-completed';
    public const ERROR_CONFIG = self::PLUGIN_ID . '-error';

    public static function init()
    {
        return new self();
    }

    public function __construct()
    {
        if (is_admin() && is_user_logged_in()) {
            wp_enqueue_script('jquery');
            add_action('admin_menu', [$this, 'setPluginMenu']);
            add_action('admin_init', [$this, 'setImagefluxCSS']);
            add_action('admin_init', [$this, 'saveBasicConfig']);
            add_action('admin_init', [$this, 'updateBasicConfig']);
            add_action('admin_init', [$this, 'saveDiffConfig']);
            add_action('admin_init', [$this, 'saveOriginSelect']);
            add_action('admin_init', [$this, 'deleteOrigin']);
            add_action('admin_init', [$this, 'saveToken']);

        }

    }

    public function setImagefluxCSS()
    {

        $plugin_url = plugin_dir_url( __FILE__ );
        wp_enqueue_style('imageflux_style', $plugin_url . 'css/imageflux-style.css' );
    }

    /**
     * メニューの設定
     */
    public function setPluginMenu()
    {

        // TOP
        add_menu_page(
            'ImageFlux',
            'ImageFlux',
            'manage_options',
            'image-flux-top',
            [$this, 'showTokenConfig'],
            'dashicons-images-alt',
            99
        );
        // オリジン一覧
        add_submenu_page(
            'image-flux-top',
            'オリジン設定',
            'オリジン設定',
            'manage_options',
            'image-flux-list',
            [$this, 'originSettings']
        );
        // 差別化機能
        add_submenu_page(
            'image-flux-top',
            '画像変換・配信設定',
            '画像変換・配信設定',
            'manage_options',
            'image-flux-diff',
            [$this, 'showDiffConfig']
        );
        // ダッシュボード
        add_submenu_page(
            'image-flux-top',
            'ダッシュボード',
            'ダッシュボード',
            'manage_options',
            'image-flux-token',
            [$this, 'showAboutPlugin']
        );
    }

    /**
     * プラグイン説明の画面を表示する
     * @throws Exception
     */
    public function showAboutPlugin()
    {

        $date_format = 'Y-m-d\T00:00:00\Z';

        $last_date = date_i18n($date_format, strtotime('first day of previous month'));
        $from_date = date_i18n($date_format, strtotime( 'first day of previous month', strtotime($last_date)));
        $to_date = date_i18n($date_format, strtotime('first day of next month'));

        $basic = new ImagefluxBasicSetting();
        $token = $basic->get('api_token');

        $image_flux = new ImageFlux($token);

        $transfer_data_origins = $image_flux->getTransferDataByDatePeriod(
            $from_date, $to_date
        );

        $transfer_view_datas = [];

        foreach ($transfer_data_origins['invoices'] as $transfer_data) {

            $month = mb_substr($transfer_data['date'], 0, 7);
            $day = (int)mb_substr($transfer_data['date'], 8, 2);

            if (empty($transfer_view_datas[$month])) {
                $transfer_view_datas[$month] = [
                    'requests' => 0,
                    'hits' => 0,
                    'miss' => 0,
                    'outboundBytes' => 0,
                    'per_hits' => 0,
                    'day' => [
                        'hits' => [],
                        'miss' => []
                    ]
                ];
            }

            $transfer_view_datas[$month]['requests'] += (int)$transfer_data['edgeStats']['requests'];
            $transfer_view_datas[$month]['hits'] += (int)$transfer_data['edgeStats']['cacheHits'];
            $transfer_view_datas[$month]['outboundBytes'] += (int)$transfer_data['edgeStats']['outboundBytes'];
            $transfer_view_datas[$month]['miss'] += (int)$transfer_data['edgeStats']['requests'] - $transfer_data['edgeStats']['cacheHits'];
            $transfer_view_datas[$month]['day']['hits'][$day] = (int)$transfer_data['edgeStats']['cacheHits'];
            $transfer_view_datas[$month]['day']['miss'][$day] = $transfer_data['edgeStats']['requests'] - $transfer_data['edgeStats']['cacheHits'];
            $transfer_view_datas[$month]['day']['outboundBytes'][$day] = (int)$transfer_data['edgeStats']['outboundBytes'];

            $graph_month = $month;

        }

        foreach ($transfer_view_datas as $month => $transfer_view_data) {

            if ($transfer_view_data['hits'] > 0) {
                $transfer_view_datas[$month]['per_hits'] = floor($transfer_view_data['hits'] / $transfer_view_data['requests'] * 100);
            }  else {
                $transfer_view_datas[$month]['per_hits'] = 0;
            }

            $transfer_view_datas[$month]['outboundBytes'] = TransferCalc::transformDigit($transfer_view_datas[$month]['outboundBytes']);
        }

        include __DIR__ . '/templates/dashboard.php';
    }

    /**
     * 基本設定の画面を表示する
     */
    public function showBasicConfig()
    {
        $basic = new ImagefluxBasicSetting();

        include __DIR__ . '/templates/basic-settings.php';
    }

    /**
     * 差別化設定の画面を表示する
     */
    public function showDiffConfig()
    {
        $diff = new ImagefluxDiffSetting();

        include __DIR__ . '/templates/diff-settings.php';
    }

    /**
     * トークン設定の画面を表示する
     */
    public function showTokenConfig()
    {
        $basic = new ImagefluxBasicSetting();

        include __DIR__ . '/templates/token-settings.php';
    }

    public function originSettings()
    {

        if (isset($_GET['origin_id'])) {
            $origin_id = sanitize_key($_GET['origin_id']);
        } else {
            $origin_id = null;
        }

        if ($origin_id === null) {
            $this->showOriginList();
        } else if ($origin_id > 0) {
            $this->editOrigin($origin_id);
        } else if ($origin_id == 0) {
            $this->showBasicConfig();
        } else {
            $this->showOriginList();
        }
    }

    /**
     * オリジンリストの表示
     */
    public function showOriginList()
    {

        $basic = new ImagefluxBasicSetting();
        $token = $basic->get('api_token');

        $image_flux = new ImageFlux($token);

        $origin_option = new ImageFluxOrigin();

        $response = $image_flux->getOriginList();

        if ($response['ok']) {
            $origin_list = $response['origins'];
        } else {
            $origin_list = [];
        }

        include __DIR__ . '/templates/origin-list.php';
    }

    public function editOrigin($origin_id)
    {

        $basic = new ImagefluxBasicSetting();
        $token = $basic->get('api_token');

        $image_flux = new ImageFlux($token);
        $response = $image_flux->getOriginInfo($origin_id);

        if (!$response['ok']) {
            error_log(json_encode($response));

            return $this->handleIfError("image-flux-list", "オリジン情報を取得できませんでした。入力内容を確認し再度お試しください。");
        }

        include __DIR__ . '/templates/basic-update-settings.php';
    }

    /**
     * 基本設定の項目をデータベースに保存する
     *
     * @return bool
     * @throws Exception
     */
    public function saveBasicConfig(): bool
    {
        // Post送信じゃない場合はスルー
        if (!$this->isPost(self::CREDENTIAL_ACTION_BASIC, self::CREDENTIAL_NAME_BASIC)) {
            return true;
        }

        $basic = new ImagefluxBasicSetting();
        $token = $basic->get('api_token');

        // オリジン発行
        $image_flux = new ImageFlux($token);

        $save_data = [
            'scheme' => sanitize_text_field($_POST['scheme']),
            'kind' => sanitize_text_field($_POST['kind']),
            'host_name' => sanitize_text_field($_POST['host_name']),
            'access_key_id' => sanitize_text_field($_POST['access_key_id']),
            'secret_access_key' => sanitize_text_field($_POST['secret_access_key']),
            'region' => sanitize_text_field($_POST['region']),
            'bucket' => sanitize_text_field($_POST['bucket']),
            'path_prefix' => sanitize_text_field($_POST['path_prefix']),
            'enable_signed_url' => sanitize_text_field($_POST['enable_signed_url']),
            'signing_secret' => sanitize_text_field($_POST['signing_secret']),
            'end_point' => sanitize_text_field($_POST['end_point']),
        ];

        $response = $image_flux->addOrigin($save_data);

        if (!$response['ok']) {
            error_log(json_encode($response));

            return $this->handleIfError("image-flux-list", "オリジンの発行に失敗しました。入力内容を確認し再度お試しください。");
        }

        $origin_id = $response['origin']['originID'];
        $origin_url = "https://" . $response['origin']['originName']['originName'];
        $organization_id = $response['origin']['organization']['organizationID'];

        $origin = new ImageFluxOrigin();
        $origin->fillSet(compact('origin_id', 'origin_url', 'organization_id'));

        $basic->fillSet($save_data);

        return $this->afterSaveSettings("image-flux-list", "基本設定の保存が完了しました。");
    }

    /**
     * 基本設定の編集内容をデータベースに保存する
     *
     * @return bool
     * @throws Exception
     */
    public function updateBasicConfig(): bool
    {
        // Post送信じゃない場合はスルー
        if (!$this->isPost(self::CREDENTIAL_ACTION_UPDATE_BASIC, self::CREDENTIAL_NAME_UPDATE_BASIC)) {
            return true;
        }

        $basic = new ImagefluxBasicSetting();
        $token = $basic->get('api_token');

        // オリジン発行
        $image_flux = new ImageFlux($token);

        $save_data = [
            'origin_id' => sanitize_text_field($_POST['origin_id']),
            'scheme' => sanitize_text_field($_POST['scheme']),
            'kind' => sanitize_text_field($_POST['kind']),
            'host_name' => sanitize_text_field($_POST['host_name']),
            'access_key_id' => sanitize_text_field($_POST['access_key_id']),
            'secret_access_key' => sanitize_text_field($_POST['secret_access_key']),
            'region' => sanitize_text_field($_POST['region']),
            'bucket' => sanitize_text_field($_POST['bucket']),
            'path_prefix' => sanitize_text_field($_POST['path_prefix']),
            'enable_signed_url' => sanitize_text_field($_POST['enable_signed_url']),
            'signing_secret' => sanitize_text_field($_POST['signing_secret']),
            'end_point' => sanitize_text_field($_POST['end_point']),
        ];

        $response = $image_flux->updateOrigin($save_data);

        // 保存成功時はnull返却
        if (!$response['ok']) {
            error_log(json_encode($response));

            return $this->handleIfError("image-flux-list", "オリジンの設定変更に失敗しました。入力内容を確認し再度お試しください。");
        }

        $origin = new ImageFluxOrigin();
        $origin_id = $response['origin']['originID'];

        // 選択中のものだった場合だけ、更新内容を反映
        if ($origin_id == $origin->get('origin_id')) {
            $basic->fillSet($save_data);
        }

        return $this->afterSaveSettings("image-flux-list", "オリジンの設定変更が完了しました。");
    }

    /**
     * 差別化設定の項目をデータベースに保存する
     *
     * @return bool
     * @throws Exception
     */
    public function saveDiffConfig(): bool
    {
        // 保存処理
        if (!$this->isPost(self::CREDENTIAL_ACTION_DIFF, self::CREDENTIAL_NAME_DIFF)) {
            return true;
        }

        $diff = new ImagefluxDiffSetting();

        $save_data = [
            'format' => sanitize_text_field($_POST['format']),
            'stream_not_allow_extensions' => sanitize_text_field($_POST['stream_not_allow_extensions']),
            'allow_extension_jpeg' => sanitize_text_field($_POST['allow_extension_jpeg']),
            'allow_extension_png' => sanitize_text_field($_POST['allow_extension_png']),
            'allow_extension_gif' => sanitize_text_field($_POST['allow_extension_gif']),
            'is_auto_resize' => sanitize_text_field($_POST['is_auto_resize']),
            'quality' => sanitize_text_field($_POST['quality']),
            'exif' => sanitize_text_field($_POST['exif']),
        ];

        $diff->fillSet($save_data);

        return $this->afterSaveSettings("image-flux-diff", "画像変換・配信設定の保存が完了しました。");

    }

    /**
     * トークンの設定を保存する
     *
     * @return bool
     * @throws Exception
     */
    public function saveToken(): bool
    {
        // 保存処理
        if (!$this->isPost(self::CREDENTIAL_ACTION_TOKEN, self::CREDENTIAL_NAME_TOKEN)) {
            return true;
        }

        $basic = new ImagefluxBasicSetting();

        $token = sanitize_text_field($_POST['api_token']);

        $basic->set('api_token', $token);

        return $this->afterSaveSettings("image-flux-top", "トークンの保存が完了しました。");
    }

    /**
     * オリジンの選択
     * @return bool
     * @throws Exception
     */
    public function saveOriginSelect(): bool
    {

        // Post送信じゃない場合はスルー
        if (!$this->isPost(self::CREDENTIAL_ACTION_ORIGIN_SELECT, self::CREDENTIAL_NAME_ORIGIN_SELECT)) {
            return true;
        }

        $basic = new ImagefluxBasicSetting();
        $token = $basic->get('api_token');

        $image_flux = new ImageFlux($token);
        $select_origin_id = sanitize_text_field($_POST['select-origin-id']);
        $response = $image_flux->getOriginInfo($select_origin_id);

        if (!$response['ok']) {
            error_log(json_encode($response));

            return $this->handleIfError("image-flux-list", "オリジン情報を取得できませんでした。入力内容を確認し再度お試しください。");
        }

        $origin_id = $response['origin']['originID'];
        $origin_url = "https://" . $response['origin']['originName']['originName'];
        $organization_id = $response['origin']['organization']['organizationID'];

        $origin = new ImageFluxOrigin();
        $origin->fillSet(compact('origin_id', 'origin_url', 'organization_id'));

        $basic_info_params = [];
        // kindで使用パラメータ変更
        if ($response['origin']['kind'] === 0) {
            // HTTPの場合
            $basic_info_params = [
                'kind' => $response['origin']['kind'],
                'scheme' => $response['origin']['config']['scheme'],
                'host_name' => $response['origin']['config']['host'],
                'path_prefix' => $response['origin']['config']['pathPrefix'],
                'access_key_id' => "",
                'secret_access_key' => "",
                'region' => "",
                'bucket' => "",
                'enable_signed_url' => $response['origin']['enableSignedUrl'],
                'signing_secret' => $response['origin']['signingSecret'],
            ];
        } else if ($response['origin']['kind'] === 1) {
            // S3の場合
            $basic_info_params = [
                'kind' => $response['origin']['kind'],
                'scheme' => "",
                'host_name' => "",
                'path_prefix' => "",
                'access_key_id' => $response['origin']['config']['accessKeyID'],
                'secret_access_key' => $response['origin']['config']['secretAccessKey'],
                'region' => $response['origin']['config']['region'],
                'bucket' => $response['origin']['config']['bucket'],
                'enable_signed_url' => $response['origin']['enableSignedUrl'],
                'signing_secret' => $response['origin']['signingSecret'],
            ];
        }

        $basic = new ImagefluxBasicSetting();
        $basic->fillSet($basic_info_params);

        return $this->afterSaveSettings("image-flux-list", "使用するオリジンの指定が完了しました。");
    }

    public function deleteOrigin(): bool
    {
        // Post送信じゃない場合はスルー
        if (!$this->isPost(self::CREDENTIAL_ACTION_ORIGIN_DELETE, self::CREDENTIAL_NAME_ORIGIN_DELETE)) {
            return true;
        }

        $basic = new ImagefluxBasicSetting();
        $token = $basic->get('api_token');

        $image_flux = new ImageFlux($token);
        $select_origin_id = sanitize_text_field($_POST['select-origin-id']);
        $response = $image_flux->deleteOrigin($select_origin_id);

        return $this->afterSaveSettings("image-flux-list", "オリジンの削除が完了しました。");
    }

    /**
     * POST送信が行われたか
     *
     * @param $nonce_action
     * @param $nonce_key
     * @return bool
     */
    private function isPost($nonce_action, $nonce_key): bool
    {
        return isset($_POST[$nonce_key]) && check_admin_referer($nonce_action, $nonce_key);
    }

    /**
     * 設定を保存し終わったあとの処理
     * @param $redirect_to_slug
     * @param $message
     * @return bool
     */
    private function afterSaveSettings($redirect_to_slug, $message): bool
    {
        // 保存が完了したら一度だけメッセージを表示する
        set_transient(self::COMPLETE_CONFIG, $message, 5);

        // 設定画面にリダイレクト
        return wp_safe_redirect(menu_page_url($redirect_to_slug));
    }

    /**
     * エラー時ハンドリング
     *
     * @param $redirect_to_slug
     * @param $message
     * @return bool
     */
    private function handleIfError($redirect_to_slug, $message): bool
    {
        set_transient(self::ERROR_CONFIG, $message, 5);

        return wp_safe_redirect(menu_page_url($redirect_to_slug));
    }

    /**
     * フルURLを返す
     *
     * @return string
     */
    public static function getFullUrl(): string
    {
        $basic = new ImagefluxBasicSetting();

        $scheme = $basic->get('scheme');
        $host_name = $basic->get('host_name');

        if (mb_substr($host_name, -1) === '/') {
            $host_name = mb_substr($host_name, 0, -1);
        }

        return $scheme . '://' . $host_name;
    }

    /**
     * オリジンURLを返す
     *
     * @return string
     */
    public static function getImageFluxOriginUrl(): string
    {
        $origin = new ImageFluxOrigin();

        return $origin->get('origin_url');
    }

    /**
     * 設定したオプションを取得する
     *
     * @param $name
     * @return mixed|void
     */
    public static function getOption($name)
    {
        $setting = new ImagefluxSetting();

        return $setting->get($name);
    }
}

/**
 * 記事を加工するクラス
 */
class ImageFluxPage
{
    /**
     * 記事を加工する
     *
     * @param $content
     * @return string|string[]|null
     */
    public static function doProcess($content)
    {
        return self::replaceImageTag($content);
    }

    /**
     * 画像タグを加工する
     *
     * @param $content
     * @return string|string[]|null
     */
    public static function replaceImageTag($content)
    {
        $basic = new ImagefluxBasicSetting();

        $kind = (int)$basic->get('kind');
        // 種別判定
        if ($kind === 0) {
            // HTTP

            preg_match_all('/<img.*?src\s*=\s*[\"|\'](.*?)[\"|\'].*?>/i', $content, $images);

            list($image_tags, $image_srcs) = $images;

            $this_url = ImageFluxManagement::getFullUrl();
            $origin_url = ImageFluxManagement::getImageFluxOriginUrl();

            $params = self::makeUrlParams();

            foreach ($image_tags as $index => $image_tag) {
                // srcの置換
                $image_src = $image_srcs[$index];

                $file_path = str_replace($this_url, '', $image_src);

                if ($file_path === $image_src) {
                    continue;
                }

                $file_path = str_replace(['https://', 'http://'], '', $file_path);
                // 署名
                $file_path = self::addSignature("/c", $params, $file_path);

                $imageflux_src = $origin_url . $file_path;

                $pattern = '/<img([^>]*)src=[\"|\']' . str_replace('/', '\/', $image_src) . '[\"|\']/';

                $replace = '<img$1src="' . $imageflux_src . '"';
                $content = preg_replace($pattern, $replace, $content);
            }
        } else if ($kind === 1){
            // S3

            preg_match_all('/<img.*?src\s*=\s*[\"|\'](.*?)[\"|\'].*?>/i', $content, $images);

            list($image_tags, $image_srcs) = $images;

            $s3_url = $basic->get('bucket') . ".s3." . $basic->get('region') . ".amazonaws.com";

            $origin_url = ImageFluxManagement::getImageFluxOriginUrl();

            $params = self::makeUrlParams();

            foreach ($image_tags as $index => $image_tag) {
                // srcの置換
                $image_src = $image_srcs[$index];

                $file_path = str_replace($s3_url, '', $image_src);

                if ($file_path === $image_src) {
                    continue;
                }

                $file_path = str_replace(['https://', 'http://'], '', $file_path);
                // 署名
                $file_path = self::addSignature("/c", $params, $file_path);

                $imageflux_src = $origin_url . $file_path;

                $pattern = '/<img([^>]*)src=[\"|\']' . str_replace('/', '\/', $image_src) . '[\"|\']/';

                $replace = '<img$1src="' . $imageflux_src . '"';
                $content = preg_replace($pattern, $replace, $content);
            }

        }

        return $content;
    }

    public static function addSignature($prefix, $params, $file_path) {
        $basic = new ImagefluxBasicSetting();
        $enable_signed_url = $basic->get('enable_signed_url');

        $path = $prefix . $params . $file_path;

        if ($enable_signed_url) {
            $signing_secret = $basic->get('signing_secret');

            $hash = hash_hmac("sha256", $path, $signing_secret, true);
            // params先頭の「/」を「,」に置き換え
            $params = substr_replace($params, ",", 0,1);

            return $prefix . "/sig=1." . rtrim(strtr(base64_encode($hash), "+/", "-_"), "="). $params . $file_path;
        }

        return $path;
    }

    /**
     * オプションの変換パラメータを生成する
     *
     * @return string
     */
    public static function makeUrlParams()
    {
        $params_str = "/";

        // フォーマット変換
        $format = ImageFluxManagement::getOption('format');

        if ($format !== null && $format !== "") {
            $params_str .= "f=" . $format . ",";
        }

        // クオリティ
        $quality = ImageFluxManagement::getOption('quality');

        if ($quality !== null && $quality !== "") {
            $params_str .= "q=" . $quality . ",";
        }

        // Exif
        $exif = ImageFluxManagement::getOption('exif');

        if ($exif !== null && $exif !== "") {
            $params_str .= "s=" . $exif . ",";
        }

        // リサイズ
        $is_auto_resize = ImageFluxManagement::getOption('is_auto_resize');

        if ($is_auto_resize === null || (int)$is_auto_resize === 1) {
            $width = self::isMobile() ? 750 : 1024;

            $params_str .= "w=" . $width . ",";
        }

        // 配信しない拡張子
        $allow_extension_jpeg = ImageFluxManagement::getOption('allow_extension_jpeg');
        $allow_extension_png = ImageFluxManagement::getOption('allow_extension_png');
        $allow_extension_gif = ImageFluxManagement::getOption('allow_extension_gif');

        $extension_param = "";
        if ((int) $allow_extension_jpeg === 1) {
            $extension_param .= "jpeg";
        }

        if ((int) $allow_extension_png === 1) {
            if ($extension_param !== "") {
                $extension_param .= ":";
            }
            $extension_param .= "png";
        }

        if ((int) $allow_extension_gif === 1) {
            if ($extension_param !== "") {
                $extension_param .= ":";
            }
            $extension_param .= "gif";
        }

        if ($extension_param !== "") {

            $params_str .= "through=" . $extension_param;
        }

        // 最後の,をカット
        if (mb_substr($params_str, -1) === ',') {
            $params_str = mb_substr($params_str, 0, -1);
        }

        return $params_str;
    }

    /**
     * モバイル端末かどうか判定する
     *
     * @return bool
     */
    public static function isMobile()
    {
        $user_agents = [
            'iPad',
            'iPhone',
            'iPod',
            'Android',
            'dream',
            'CUPCAKE',
            'Windows',
            'blackberry',
            'webOS',
            'incognito',
            'webmate',
        ];

        $pattern = '/' . implode('|', $user_agents) . '/i';

        return preg_match($pattern, $_SERVER['HTTP_USER_AGENT']) === 1;
    }
}

?>
